@@ -17,6 +17,9 @@
 #include "../crc/sha256.h"
 #include "../crc/sha512.h"
 #include "../crc/xxhash.h"
+#include "../crc/murmur3.h"
+#include "../crc/fnv.h"
+#include "../hash.h"
 
 #include "test.h"
 
@@ -26,7 +29,8 @@
 struct test_type {
 	const char *name;
 	unsigned int mask;
-	void (*fn)(void *, size_t);
+	void (*fn)(struct test_type *, void *, size_t);
+	uint32_t output;
 };
 
 enum {
@@ -40,9 +44,12 @@ enum {
 	T_SHA256	= 1U << 7,
 	T_SHA512	= 1U << 8,
 	T_XXHASH	= 1U << 9,
+	T_MURMUR3	= 1U << 10,
+	T_JHASH		= 1U << 11,
+	T_FNV		= 1U << 12,
 };
 
-static void t_md5(void *buf, size_t size)
+static void t_md5(struct test_type *t, void *buf, size_t size)
 {
 	uint32_t digest[4];
 	struct fio_md5_ctx ctx = { .hash = digest };
@@ -50,11 +57,13 @@ static void t_md5(void *buf, size_t size)
 
 	fio_md5_init(&ctx);
 
-	for (i = 0; i < NR_CHUNKS; i++)
+	for (i = 0; i < NR_CHUNKS; i++) {
 		fio_md5_update(&ctx, buf, size);
+		fio_md5_final(&ctx);
+	}
 }
 
-static void t_crc64(void *buf, size_t size)
+static void t_crc64(struct test_type *t, void *buf, size_t size)
 {
 	int i;
 
@@ -62,7 +71,7 @@ static void t_crc64(void *buf, size_t size)
 		fio_crc64(buf, size);
 }
 
-static void t_crc32(void *buf, size_t size)
+static void t_crc32(struct test_type *t, void *buf, size_t size)
 {
 	int i;
 
@@ -70,7 +79,7 @@ static void t_crc32(void *buf, size_t size)
 		fio_crc32(buf, size);
 }
 
-static void t_crc32c(void *buf, size_t size)
+static void t_crc32c(struct test_type *t, void *buf, size_t size)
 {
 	int i;
 
@@ -78,7 +87,7 @@ static void t_crc32c(void *buf, size_t size)
 		fio_crc32c(buf, size);
 }
 
-static void t_crc16(void *buf, size_t size)
+static void t_crc16(struct test_type *t, void *buf, size_t size)
 {
 	int i;
 
@@ -86,7 +95,7 @@ static void t_crc16(void *buf, size_t size)
 		fio_crc16(buf, size);
 }
 
-static void t_crc7(void *buf, size_t size)
+static void t_crc7(struct test_type *t, void *buf, size_t size)
 {
 	int i;
 
@@ -94,7 +103,7 @@ static void t_crc7(void *buf, size_t size)
 		fio_crc7(buf, size);
 }
 
-static void t_sha1(void *buf, size_t size)
+static void t_sha1(struct test_type *t, void *buf, size_t size)
 {
 	uint32_t sha[5];
 	struct fio_sha1_ctx ctx = { .H = sha };
@@ -106,7 +115,7 @@ static void t_sha1(void *buf, size_t size)
 		fio_sha1_update(&ctx, buf, size);
 }
 
-static void t_sha256(void *buf, size_t size)
+static void t_sha256(struct test_type *t, void *buf, size_t size)
 {
 	uint8_t sha[64];
 	struct fio_sha256_ctx ctx = { .buf = sha };
@@ -114,11 +123,13 @@ static void t_sha256(void *buf, size_t size)
 
 	fio_sha256_init(&ctx);
 
-	for (i = 0; i < NR_CHUNKS; i++)
+	for (i = 0; i < NR_CHUNKS; i++) {
 		fio_sha256_update(&ctx, buf, size);
+		fio_sha256_final(&ctx);
+	}
 }
 
-static void t_sha512(void *buf, size_t size)
+static void t_sha512(struct test_type *t, void *buf, size_t size)
 {
 	uint8_t sha[128];
 	struct fio_sha512_ctx ctx = { .buf = sha };
@@ -130,7 +141,31 @@ static void t_sha512(void *buf, size_t size)
 		fio_sha512_update(&ctx, buf, size);
 }
 
-static void t_xxhash(void *buf, size_t size)
+static void t_murmur3(struct test_type *t, void *buf, size_t size)
+{
+	int i;
+
+	for (i = 0; i < NR_CHUNKS; i++)
+		murmurhash3(buf, size, 0x8989);
+}
+
+static void t_jhash(struct test_type *t, void *buf, size_t size)
+{
+	int i;
+
+	for (i = 0; i < NR_CHUNKS; i++)
+		t->output += jhash(buf, size, 0x8989);
+}
+
+static void t_fnv(struct test_type *t, void *buf, size_t size)
+{
+	int i;
+
+	for (i = 0; i < NR_CHUNKS; i++)
+		t->output += fnv(buf, size, 0x8989);
+}
+
+static void t_xxhash(struct test_type *t, void *buf, size_t size)
 {
 	void *state;
 	int i;
@@ -140,7 +175,7 @@ static void t_xxhash(void *buf, size_t size)
 	for (i = 0; i < NR_CHUNKS; i++)
 		XXH32_update(state, buf, size);
 
-	XXH32_digest(state);
+	t->output = XXH32_digest(state);
 }
 
 static struct test_type t[] = {
@@ -194,6 +229,21 @@ static struct test_type t[] = {
 		.mask = T_XXHASH,
 		.fn = t_xxhash,
 	},
+	{
+		.name = "murmur3",
+		.mask = T_MURMUR3,
+		.fn = t_murmur3,
+	},
+	{
+		.name = "jhash",
+		.mask = T_JHASH,
+		.fn = t_jhash,
+	},
+	{
+		.name = "fnv",
+		.mask = T_FNV,
+		.fn = t_fnv,
+	},
 	{
 		.name = NULL,
 	},
@@ -261,6 +311,7 @@ int fio_crctest(const char *type)
 		struct timeval tv;
 		double mb_sec;
 		uint64_t usec;
+		char pre[3];
 
 		if (!(t[i].mask & test_mask))
 			continue;
@@ -271,16 +322,20 @@ int fio_crctest(const char *type)
 		 */
 		if (first) {
 			usec_spin(100000);
-			t[i].fn(buf, CHUNK);
+			t[i].fn(&t[i], buf, CHUNK);
 		}
 
 		fio_gettime(&tv, NULL);
-		t[i].fn(buf, CHUNK);
+		t[i].fn(&t[i], buf, CHUNK);
 		usec = utime_since_now(&tv);
 
 		mb_sec = (double) mb / (double) usec;
 		mb_sec /= (1.024 * 1.024);
-		printf("%s:\t%8.2f MB/sec\n", t[i].name, mb_sec);
+		if (strlen(t[i].name) >= 7)
+			sprintf(pre, "\t");
+		else
+			sprintf(pre, "\t\t");
+		printf("%s:%s%8.2f MB/sec\n", t[i].name, pre, mb_sec);
 		first = 0;
 	}
 
