@@ -10,9 +10,21 @@ enum fio_ddir {
 	DDIR_DATASYNC,
 	DDIR_SYNC_FILE_RANGE,
 	DDIR_WAIT,
+	DDIR_LAST,
 	DDIR_INVAL = -1,
 };
 
+static inline const char *io_ddir_name(enum fio_ddir ddir)
+{
+	const char *name[] = { "read", "write", "trim", "sync", "datasync",
+				"sync_file_range", "write", };
+
+	if (ddir < DDIR_LAST)
+		return name[ddir];
+
+	return "invalid";
+}
+
 enum td_ddir {
 	TD_DDIR_READ		= 1 << 0,
 	TD_DDIR_WRITE		= 1 << 1,
@@ -43,6 +55,15 @@ static inline int ddir_rw(enum fio_ddir ddir)
 	return ddir == DDIR_READ || ddir == DDIR_WRITE || ddir == DDIR_TRIM;
 }
 
+static inline const char *ddir_str(enum td_ddir ddir)
+{
+	const char *ddir_str[] = { NULL, "read", "write", "rw", NULL,
+				   "randread", "randwrite", "randrw",
+				   "trim", NULL, NULL, NULL, "randtrim" };
+
+	return ddir_str[ddir];
+}
+
 #define ddir_trim(ddir) ((ddir) == DDIR_TRIM)
 
 #define ddir_rw_sum(arr)	\
