@@ -5,7 +5,7 @@
  * nothing to do with anything remotely narcissistic.
  *
  * A set bit at layer N indicates a full word at layer N-1, and so forth. As
- * the bitmap becomes progressively more full, checking for existance
+ * the bitmap becomes progressively more full, checking for existence
  * becomes cheaper (since fewer layers are walked, making it a lot more
  * cache friendly) and locating the next free space likewise.
  *
@@ -33,7 +33,7 @@
 #error "Number of arch bits unknown"
 #endif
 
-#define BLOCKS_PER_UNIT		(1UL << UNIT_SHIFT)
+#define BLOCKS_PER_UNIT		(1U << UNIT_SHIFT)
 #define BLOCKS_PER_UNIT_MASK	(BLOCKS_PER_UNIT - 1)
 
 #define firstfree_valid(b)	((b)->first_free != (uint64_t) -1)
@@ -189,7 +189,6 @@ void axmap_clear(struct axmap *axmap, uint64_t bit_nr)
 struct axmap_set_data {
 	unsigned int nr_bits;
 	unsigned int set_bits;
-	unsigned int fail_ok;
 };
 
 static unsigned long bit_masks[] = {
@@ -229,10 +228,8 @@ static int axmap_set_fn(struct axmap_level *al, unsigned long offset,
 	 * Mask off any potential overlap, only sets contig regions
 	 */
 	overlap = al->map[offset] & mask;
-	if (overlap == mask) {
-		assert(data->fail_ok);
+	if (overlap == mask)
 		return 1;
-	}
 
 	while (overlap) {
 		unsigned long clear_mask = ~(1UL << ffz(~overlap));
@@ -273,14 +270,14 @@ static void __axmap_set(struct axmap *axmap, uint64_t bit_nr,
 		axmap_handler(axmap, bit_nr, axmap_set_fn, data);
 		set_bits += data->set_bits;
 
-		if (data->set_bits != (BLOCKS_PER_UNIT - nr_bits))
+		if (!data->set_bits ||
+		    data->set_bits != (BLOCKS_PER_UNIT - nr_bits))
 			break;
 
 		nr_bits -= data->set_bits;
 		bit_nr += data->set_bits;
 
 		data->nr_bits = nr_bits;
-		data->fail_ok = 1;
 	}
 
 	data->set_bits = set_bits;
@@ -295,10 +292,27 @@ void axmap_set(struct axmap *axmap, uint64_t bit_nr)
 
 unsigned int axmap_set_nr(struct axmap *axmap, uint64_t bit_nr, unsigned int nr_bits)
 {
-	struct axmap_set_data data = { .nr_bits = nr_bits, };
+	unsigned int set_bits = 0;
 
-	__axmap_set(axmap, bit_nr, &data);
-	return data.set_bits;
+	do {
+		struct axmap_set_data data = { .nr_bits = nr_bits, };
+		unsigned int max_bits, this_set;
+
+		max_bits = BLOCKS_PER_UNIT - (bit_nr & BLOCKS_PER_UNIT_MASK);
+		if (max_bits < nr_bits)
+			data.nr_bits = max_bits;
+
+		this_set = data.nr_bits;
+		__axmap_set(axmap, bit_nr, &data);
+		set_bits += data.set_bits;
+		if (data.set_bits != this_set)
+			break;
+
+		nr_bits -= data.set_bits;
+		bit_nr += data.set_bits;
+	} while (nr_bits);
+
+	return set_bits;
 }
 
 static int axmap_isset_fn(struct axmap_level *al, unsigned long offset,
